package wordcheck;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

public class WordProcessorTest {
  private WordProcessor wordProcessor;

  @Test
  void canary(){
    assertTrue(true);
  }

  @BeforeEach
  void init() {
    SpellChecker spellChecker = word -> true;
    wordProcessor = new WordProcessor(spellChecker);
  }

  @Test
  void processTextWithCorrectSpelling(){
    assertAll(
      () -> assertEquals("", wordProcessor.processText("")),
      () -> assertEquals("good", wordProcessor.processText("good")),
      () -> assertEquals("blah", wordProcessor.processText("blah"))
    );
  }

  @Test
  void processTextThatIsConsideredIncorrectSpelling(){
    wordProcessor = new WordProcessor(word -> false);

    assertEquals("_blah_", wordProcessor.processText("blah"));
  }

  @Test
  void processTextStringWithCorrectSpellings(){
    assertEquals("how are you", wordProcessor.processText("how are you"));
  }

  @Test
  void processTextStringWithOneWordConsideredIncorrectSpelling(){
    wordProcessor = new WordProcessor(word -> !word.equals("are"));

    assertEquals("how _are_ you", wordProcessor.processText("how are you"));
  }

  @Test
  void processTextStringWithTwoWordsConsideredIncorrectSpelling(){
    var incorrectSpellings = List.of("how", "are");
    wordProcessor = new WordProcessor(word -> !incorrectSpellings.contains(word));

    assertEquals("hello there _how_ _are_ you", wordProcessor.processText("hello there how are you"));
  }

  private boolean stubSpellCheckerWithFailure(String word) {
    if(word.equals("are")) {
      throw new RuntimeException("Network Failure");
    }

    return true;
  }

  @Test
  void processTextStringWithFailureToCheckOneWord(){
    wordProcessor = new WordProcessor(this::stubSpellCheckerWithFailure);

    assertEquals("how ~are~ you", wordProcessor.processText("how are you"));
  }
}
