package wordcheck;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockserver.verify.VerificationTimes;
import org.testcontainers.containers.MockServerContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;
import org.testcontainers.utility.DockerImageName;
import org.mockserver.client.MockServerClient;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockserver.model.HttpRequest.request;
import static org.mockserver.model.HttpResponse.response;

@Testcontainers
public class AgileDeveloperSpellCheckTest {
  private AgileDeveloperSpellCheck agileDeveloperSpellCheck;

  @Container
  private static final MockServerContainer mockServerContainer =
    new MockServerContainer(DockerImageName.parse("mockserver/mockserver"));

  private static MockServerClient mockServerClient;

  @BeforeAll
  static void beforeAll() {
    mockServerClient = new MockServerClient(
      mockServerContainer.getHost(), mockServerContainer.getServerPort());
  }

  @BeforeEach
  void init() {
    mockServerClient.reset();

    agileDeveloperSpellCheck = new AgileDeveloperSpellCheck();
  }

  private void runTestWithMockServerClient(String word, boolean expected) {
    mockServerClient
      .when(request()
        .withPath("/spellcheck")
        .withQueryStringParameter("check", word))
      .respond(
        response().withBody(Boolean.toString(expected)));

    var url = mockServerContainer.getEndpoint();
    System.setProperty("SPELLCHECKER_URL", url);

    assertEquals(expected, agileDeveloperSpellCheck.isSpellingCorrect(word));

    mockServerClient.verify(
      request()
        .withMethod("GET")
        .withPath("/spellcheck")
        .withQueryStringParameter("check", word),
      VerificationTimes.once());
  }

  @Test
  void checkSpellingForACorrectWord(){
    runTestWithMockServerClient("good", true);
  }

  @Test
  void checkSpellingForAnotherCorrectWord(){
    runTestWithMockServerClient("nice", true);
  }

  @Test
  void checkSpellingForAIncorrectWord(){
    runTestWithMockServerClient("aer", false);
  }

  @Test
  void checkSpellingForAnotherIncorrectWord(){
    runTestWithMockServerClient("hwo", false);
  }
}
